@@ -2,7 +2,7 @@
 /*
  * FCRON - periodic command scheduler 
  *
- *  Copyright 2000-2006 Thibault Godouet <fcron@free.fr>
+ *  Copyright 2000-2007 Thibault Godouet <fcron@free.fr>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  *  `LICENSE' that comes with the fcron source distribution.
  */
 
- /* $Id: allow.c,v 1.15 2006/01/11 00:58:02 thib Exp thib $ */
+ /* $Id: allow.c,v 1.18 2007/06/03 17:53:02 thib Exp thib $ */
 
 #include "fcrontab.h"
 
@@ -75,14 +75,14 @@ in_file(char *str, char *file)
 
 int
 is_allowed(char *user)
-    /* return 1 if user is allowed to use this soft
+    /* return 1 if user is allowed to use this software
      * otherwise return 0 */
 {
     int allow = 0;
     int deny = 0;
 
     /* check if user is in passwd file */
-    if ( ! getpwnam(user) )
+    if ( getpwnam(user) == NULL )
 	return 0;
 
     /* check if user is in fcron.allow and/or in fcron.deny files */
