@@ -1,8 +1,7 @@
-
 /*
  * FCRON - periodic command scheduler 
  *
- *  Copyright 2000-2006 Thibault Godouet <fcron@free.fr>
+ *  Copyright 2000-2007 Thibault Godouet <fcron@free.fr>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,7 +21,7 @@
  *  `LICENSE' that comes with the fcron source distribution.
  */
 
- /* $Id: fcrondyn.c,v 1.14 2006/01/11 00:48:33 thib Exp thib $ */
+ /* $Id: fcrondyn.c,v 1.19 2007/04/14 18:04:11 thib Exp thib $ */
 
 /* fcrondyn : interact dynamically with running fcron process :
  *     - list jobs, with their status, next time of execution, etc
@@ -35,7 +34,7 @@
 #include "allow.h"
 #include "read_string.h"
 
-char rcs_info[] = "$Id: fcrondyn.c,v 1.14 2006/01/11 00:48:33 thib Exp thib $";
+char rcs_info[] = "$Id: fcrondyn.c,v 1.19 2007/04/14 18:04:11 thib Exp thib $";
 
 void info(void);
 void usage(void);
@@ -67,6 +66,12 @@ char foreground = 1;
 char dosyslog = 1;
 pid_t daemon_pid = 0;
 
+/* uid/gid of user/group root 
+ * (we don't use the static UID or GID as we ask for user and group names
+ * in the configure script) */
+uid_t rootuid = 0;
+gid_t rootgid = 0;
+
 /* misc */
 char *user_str;
 uid_t user_uid;
@@ -223,7 +228,7 @@ parse_cmd(char *cmd_str, long int **cmd, int *cmd_len)
 	    }
 
 	    /* use default value : currently, works only with CUR_USER */
-	    if ( user_uid == ROOTUID ) {
+	    if ( user_uid == rootuid ) {
 		/* default for root = all */
 		int_buf = ALL;
 		Write_cmd( int_buf );
@@ -291,7 +296,7 @@ parse_cmd(char *cmd_str, long int **cmd, int *cmd_len)
 	    case NICE_VALUE:
 		/* after strtol(), cmd_str will be updated (first non-number char) */
 		if ( (int_buf = strtol(cmd_str, &cmd_str, 10)) > 20 
-		     || (int_buf < 0 && getuid() != ROOTUID) || int_buf < -20
+		     || (int_buf < 0 && getuid() != rootuid) || int_buf < -20
 		     || (! isspace( (int) *cmd_str) && *cmd_str != '\0') ) {
 		    fprintf(stderr, "Error : invalid nice value.\n");
 		    return INVALID_ARG;
@@ -399,17 +404,26 @@ connect_fcron(void)
     int fd = -1;
     struct sockaddr_un addr;
     int len = 0;
+    int sun_len = 0;
 
     if ( (fd = socket(PF_UNIX, SOCK_STREAM, 0)) == -1 )
 	die_e("could not create socket");
 
     addr.sun_family = AF_UNIX;
-    if ( (len = strlen(fifofile)) > sizeof(addr.sun_path) )
-	die("Error : fifo file path too long (max is %d)", sizeof(addr.sun_path));
-    strncpy(addr.sun_path, fifofile, sizeof(addr.sun_path) - 1);
+    len = strlen(fifofile);
+    if ( len > sizeof(addr.sun_path) - 1 )
+	die("Error : fifo file path too long (max is %d)", sizeof(addr.sun_path) - 1);
+    /* length(fifofile) < sizeof(add.sun_path), so strncpy will terminate
+     * the string with at least one \0 (not necessarily required by the OS,
+     * but still a good idea) */
+    strncpy(addr.sun_path, fifofile, sizeof(addr.sun_path));
     addr.sun_path[sizeof(addr.sun_path)-1] = '\0';
+    sun_len = (addr.sun_path - (char *)&addr) + len;
+#if HAVE_SA_LEN
+    addr.sun_len = sun_len;
+#endif
 
-    if ( connect(fd, (struct sockaddr *) &addr, sizeof(addr.sun_family) + len) < 0 )
+    if ( connect(fd, (struct sockaddr *) &addr, sun_len) < 0 )
 	die_e("Cannot connect() to fcron (check if fcron is running)");
 
     if ( authenticate_user(fd) == ERR ) {
@@ -502,7 +516,7 @@ talk_fcron(char *cmd_str, int fd)
 	return ERR;
     }
 
-    while ( (read_len = recv(fd, buf, sizeof(buf), 0)) >= 0 || errno == EINTR ) {
+    while ( (read_len = (size_t)recv(fd, buf, sizeof(buf), 0)) >= 0 || errno == EINTR ) {
 	if ( errno == EINTR && debug_opt)
 	    fprintf(stderr, "got EINTR ...\n");
 	else if ( read_len > sizeof(buf) ) {
@@ -512,13 +526,14 @@ talk_fcron(char *cmd_str, int fd)
 	}
 	else if ( read_len <= 0 ) {
 	    if ( debug_opt )
-		fprintf(stderr, "read_len = %d\n", read_len);
+		fprintf(stderr, "read_len = %d\n", (int)read_len);
 	    fprintf(stderr, "connection closed by fcron\n");
 	    shutdown(fd, SHUT_RDWR);
 	    return ERR;
 	}
 	else {
-	    write(STDOUT_FILENO, buf, read_len);
+	    if ( write(STDOUT_FILENO, buf, read_len) < 0 )
+		error_e("unable to write() to STDOUT_FILENO");
 	    if ( read_len >= sizeof(END_STR) &&
 		 strncmp(&buf[read_len-sizeof(END_STR)], END_STR, sizeof(END_STR)) == 0)
 		break;
@@ -621,6 +636,9 @@ main(int argc, char **argv)
     int fd = (-1);
     struct passwd *pass = NULL;
 
+    rootuid = get_user_uid_safe(ROOTNAME);
+    rootgid = get_group_gid_safe(ROOTGROUP);
+
     if ( strrchr(argv[0], '/') == NULL) prog_name = argv[0];
     else prog_name = strrchr(argv[0], '/') + 1;
 
