@@ -1,7 +1,7 @@
 /*
  * FCRON - periodic command scheduler 
  *
- *  Copyright 2000-2006 Thibault Godouet <fcron@free.fr>
+ *  Copyright 2000-2007 Thibault Godouet <fcron@free.fr>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  *  `LICENSE' that comes with the fcron source distribution.
  */
 
- /* $Id: fcrondyn.h,v 1.5 2006/01/11 00:49:26 thib Exp thib $ */
+ /* $Id: fcrondyn.h,v 1.7 2007/04/14 18:04:11 thib Exp thib $ */
 
 #ifndef __FCRONDYN_H__
 #define __FCRONDYN_H__
@@ -33,7 +33,8 @@
 extern char debug_opt;
 extern char dosyslog;
 extern pid_t daemon_pid;
-
+extern uid_t rootuid;
+extern gid_t rootgid;
 
 /* types def */
 #define MAX_NUM_OPT 4
