@@ -21,7 +21,7 @@
  *  `LICENSE' that comes with the fcron source distribution.
  */
 
- /* $Id: option.h,v 1.25 2007/04/14 18:04:26 thib Exp thib $ */
+ /* $Id: option.h,v 1.27 2007/11/07 09:23:50 thib Exp thib $ */
 
 /* This has been inspired from bitstring(3) : here is the original copyright :
  */
@@ -52,7 +52,8 @@
 
 /* WARNING : do not change any option number, nor remove any option, nor change
  *           the meaning of a value (i.e. bit set to 1 -> true).
- *           It can lead to errors with fcrontabs save to disk/load in memory.
+ *           It can lead to errors with fcrontabs already saved to disk
+ *           or loaded in memory.
  */
 
 /*
@@ -80,13 +81,14 @@
   16     should mons field be ignored in goto_non_matching() ?
   17     should dow field be ignored in goto_non_matching() ?
   18     First freq is the freq (*ly) or the first field to take into account ?
-  19     Freq (ie daily) is from middle to middle of interval (ie nightly) ?
+  19     Freq (ie daily) is from middle to middle of period (ie nightly) ?
   20     Should we remove a %-job from lavg queue if the interval is exceeded ?
   21     Should user be mailed if a %-job has not run during a period ?
   22     Should fcron log everything about this job or just errors ?
   23     Should this job be run asap, or randomly in its allowed interval of execution ?
   24     Should first value be applied at each fcron startup, or before line first exe ?
   25     if fcron is running in foreground, print jobs output to stderr/stdout or mail ?
+  26     should the output of the job be emailed to the user only non-zero exit status ?
 
 */
 
@@ -358,8 +360,8 @@
 
 
 /*
-  bit 19 : set to 1 : run once from mid-interval to mid-interval (i.e. nightly)
-           set to 0 : run once from begin to the end of interval (i.e. daily)
+  bit 19 : set to 1 : run once from mid-period to mid-period (i.e. nightly)
+           set to 0 : run once from begin to the end of period (i.e. daily)
 */
 #define	is_freq_mid(opt) \
 	(_bit_test(opt, 19))
@@ -442,6 +444,18 @@
 #define clear_stdout(opt) \
 	(_bit_clear(opt, 25))
 
+/*
+  bit 26 : set to 1 : The ouput of a job will only be emailed to the user if the job
+                      exited with a non-zero status.
+           set to 0 : The exit-status of the job won't be taken into account to decide
+                      if the output should be emailed to the user.
+*/
+#define	is_erroronlymail(opt) \
+	(_bit_test(opt, 26))
+#define	set_erroronlymail(opt) \
+	(_bit_set(opt, 26))
+#define clear_erroronlymail(opt) \
+	(_bit_clear(opt, 26))
 
 #endif /* __OPTIONH__ */
 
